@@ -32,6 +32,8 @@
 
 #include <libtracker-extract/tracker-extract.h>
 
+#include <libtracker-miners-common/valgrind.h>
+
 #include "tracker-extract.h"
 #include "tracker-main.h"
 
@@ -264,10 +266,12 @@ notify_task_finish (TrackerExtractTask *task,
 		if (task->module) {
 			stats_data = g_hash_table_lookup (priv->statistics_data,
 			                                  task->module);
-			stats_data->extracted_count++;
+			if (stats_data) {
+				stats_data->extracted_count++;
 
-			if (!success) {
-				stats_data->failed_count++;
+				if (!success) {
+					stats_data->failed_count++;
+				}
 			}
 		} else {
 			priv->unhandled_count++;
@@ -337,7 +341,7 @@ task_deadline_cb (gpointer user_data)
 	g_warning ("File '%s' took too long to process. Shutting down everything",
 	           task->file);
 
-	_exit (EXIT_FAILURE);
+	exit (EXIT_FAILURE);
 }
 
 static TrackerExtractTask *
@@ -387,7 +391,7 @@ extract_task_new (TrackerExtract *extract,
 	task->extract = extract;
 	task->max_text = priv->max_text;
 
-	if (task->res) {
+	if (task->res && !RUNNING_ON_VALGRIND) {
 		if (deadline_seconds < 0) {
 			const gchar *deadline_envvar;
 
