@@ -263,7 +263,7 @@ class TrackerMinerFTSTest(TrackerMinerTest):
 
         self.testfile = "test-monitored/miner-fts-test.txt"
 
-    def set_text(self, text):
+    def set_text(self, text, encoding=None):
         text_escaped = Tracker.sparql_escape_string(text)
         path = pathlib.Path(self.path(self.testfile))
 
@@ -277,21 +277,30 @@ class TrackerMinerFTSTest(TrackerMinerTest):
                 f'nie:plainTextContent "{text_escaped}"',
                 timeout=cfg.AWAIT_TIMEOUT,
             ):
-                path.write_text(text)
+                if encoding:
+                    path.write_bytes(text.encode(encoding))
+                else:
+                    path.write_text(text)
         elif path.exists() and text == '':
             old_text_escaped = Tracker.sparql_escape_string(path.read_text())
             resource_id = self.tracker.get_content_resource_id(self.uri(self.testfile))
             with self.tracker.await_delete(DOCUMENTS_GRAPH,
                                            resource_id,
                                            timeout=cfg.AWAIT_TIMEOUT):
-                path.write_text(text)
+                if encoding:
+                    path.write_bytes(text.encode(encoding))
+                else:
+                    path.write_text(text)
         elif text != '':
             url = self.uri(self.testfile)
             expected = f'a nfo:Document; nie:isStoredAs <{url}>; nie:plainTextContent "{text_escaped}"'
             with self.tracker.await_insert(
                 DOCUMENTS_GRAPH, expected, timeout=cfg.AWAIT_TIMEOUT
             ):
-                path.write_text(text)
+                if encoding:
+                    path.write_bytes(text.encode(encoding))
+                else:
+                    path.write_text(text)
 
     def search_word(self, word):
         """
